<?php

// Exit if accessed directly
if ( !defined( 'ABSPATH' ) ) exit;

//if ( !class_exists('NNRobots_Content_Fire_Functions') ) :

add_action( 'the_content',								array('NNRobots_Content_Fire_Functions', 'display'));
add_action( 'wp_ajax_nnr_cf_get_styling_data', 			array('NNRobots_Content_Fire_Functions', 'get_styling_data'));
add_action( 'wp_ajax_nopriv_nnr_cf_get_styling_data', 	array('NNRobots_Content_Fire_Functions', 'get_styling_data'));

/**
 *
 * Main class used to display the content fire on the website and display backend options
 *
 * @since 1.0.0
 */
class NNRobots_Content_Fire_Functions {

	/**
	 * Hook function to wp_footer to display content fire on website
	 *
	 * @since 1.0.0
	 *
	 * @param 	N/A
	 * @return 	N/A
	 */
	static function display($content) {

		include_once(NNROBOTS_CONTENT_FIRE_PLUGIN_DIR . 'admin/model/db.php');

		global $wpdb;
		global $post;
		global $cat;

		$data = $wpdb->get_results("SELECT * FROM `" . NNRobots_Content_Fire_DB::getTableName() . "`",'ARRAY_A');

        $done = false;

        // Get all post types that are public

		$post_types = get_post_types(array('public' => true));

		$content_fire_content = '';

		// Post Types

		if (!$done && is_singular()) {

			foreach ($data as $val) {

	        	$args         = self::mb_unserialize($val['args']);
	        	$template     = self::mb_unserialize($val['template']);
        		$trigger      = self::mb_unserialize($val['trigger']);
        		$news_letter  = self::mb_unserialize($val['news_letter']);

        		// Check that conditions are true

        		if (!self::check_conditions($val['id'], $template, $args, $trigger)) {
	        		continue;
        		}

        		$post_ids = array();

        		if (isset($trigger[$post->post_type]['id'])) {
	        		$post_ids = explode(',', $trigger[$post->post_type]['id']);
        		}

	    		// Check if post types match

	    		if (isset($trigger['display_on']) && $trigger['display_on'] != $post->post_type) {
		    		continue;
	    		}

        		// ID

        		if (isset($trigger[$post->post_type]['all']) &&
        			!$trigger[$post->post_type]['all'] &&
	    			in_array($post->ID, $post_ids) &&
	    			$val['active'] == 'Active' &&
	    			!$done) {

	        		$content_fire_content = self::generate_code($val, $news_letter);
	        		$done = true;
					break;
        		}
	        }

	        if (!$done) {

				foreach ($data as $val) {

		        	$args         = self::mb_unserialize($val['args']);
		        	$template     = self::mb_unserialize($val['template']);
	        		$trigger      = self::mb_unserialize($val['trigger']);
	        		$news_letter  = self::mb_unserialize($val['news_letter']);

	        		// Check that conditions are true

	        		if (!self::check_conditions($val['id'], $template, $args, $trigger)) {
		        		continue;
	        		}

	        		$post_ids = array();

	        		if (isset($trigger[$post->post_type]['id'])) {
		        		$post_ids = explode(',', $trigger[$post->post_type]['id']);
	        		}

		    		// Check if post types match

		    		if (isset($trigger['display_on']) &&
		    			$trigger['display_on'] != $post->post_type &&
		    			$trigger['display_on'] != 'category' &&
		    			$trigger['display_on'] != 'tag') {

						continue;
			    	}

			    	// Tag ID

			    	$tags = explode(',', $trigger['tag']['id']);

	        		if ($trigger['display_on'] == 'tag' &&
	        			has_tag($tags,$post->ID) &&
	        			$val['active'] == 'Active' &&
	        			!$done) {

		        		$content_fire_content = self::generate_code($val, $news_letter);
		        		$done = true;
						break;
	        		}

	        		// Category ID

	        		if ($trigger['display_on'] == 'category' &&
	        			isset($trigger['category']['all']) &&
	        			!$trigger['category']['all'] &&
		        		$val['active'] == 'Active' &&
	        			in_category($category_ids, $post->ID) &&
	        			!$done) {

		        		$content_fire_content = self::generate_code($val, $news_letter);
		        		$done = true;
						break;
	        		}

	        		// All

	        		if ($trigger['display_on'] != 'category' &&
	        			$trigger['display_on'] != 'tag' &&
	        			isset($trigger[$post->post_type]['all']) &&
	        			$trigger[$post->post_type]['all'] &&
	        			$val['active'] == 'Active' &&
	        			!$done) {

		        		$content_fire_content = self::generate_code($val, $news_letter);
		        		$done = true;
						break;
	        		}
		        }
		    }
		}

		if ($content_fire_content != '') {

			// Top of Post

			if (isset($args['placement']['type']) && $args['placement']['type'] == 'before') {
				return $content_fire_content . $content;
			}

			// Bottom of Post

			else if (isset($args['placement']['type']) && $args['placement']['type'] == 'after') {
				return $content . $content_fire_content;
			}

			// Paragraphs

			else if (isset($args['placement']['type']) && $args['placement']['type'] == 'paragraphs') {
				return $content . $content_fire_content;
			}

			// None

			else if (isset($args['placement']['type']) && $args['placement']['type'] == 'none') {
				return $content;
			}
		}

        return $content;
	}

	/**
	 * Displays the content fire
	 *
	 * @since 1.0.0
	 *
	 * @param 	$val the array of the content fire attr
	 * @param 	$news letter the array of the content fire attr
	 * @return 	N/A
	 */
	static function generate_code($val, $news_letter, $widget = false) {

		if (!isset($_GET['action']) || (isset($_GET['action']) && $_GET['action'] != 'preview')) {
			// Check if this content fire has any A/B Testing

			$args = self::mb_unserialize($val['args']);

			if ($args['ab_testing']['active'] && $args['ab_testing']['content_fire_id'] != '0') {

				$nnr_content_fire_db 	    = new NNRobots_Content_Fire_DB();
				$ab_content_fire 			= $nnr_content_fire_db->get_data_from_id($args['ab_testing']['content_fire_id']);
				$ab_args 			= self::mb_unserialize($ab_content_fire['args']);

				// Check to make sure both content fires have A/B Testing Active and that the content fire ids match

				if (isset($ab_content_fire) &&
					$ab_args['ab_testing']['active'] &&
					$val['id'] == $ab_args['ab_testing']['content_fire_id'] &&
					$ab_content_fire['id'] == $args['ab_testing']['content_fire_id']) {

					// Value of -1 is default and will show the original optin

					if ( !isset($args['ab_testing']['next']) ) {
						$args['ab_testing']['next'] = $val['id'];
					}

					// Show A

					if ( $args['ab_testing']['next'] == $val['id'] ) {

						$args['ab_testing']['next'] = $ab_content_fire['id'];

						$val['args']         = self::mb_unserialize($args);
						$val['template']     = self::mb_unserialize($val['template']);
						$val['trigger']      = self::mb_unserialize($val['trigger']);
						$val['news_letter']  = self::mb_unserialize($val['news_letter']);
						$val['style']        = self::mb_unserialize($val['style']);

						$nnr_content_fire_db->update_data($val);
					}

					// Show B

					else {

						$args['ab_testing']['next'] = $val['id'];

						$val['args']         = self::mb_unserialize($args);
						$val['template']     = self::mb_unserialize($val['template']);
						$val['trigger']      = self::mb_unserialize($val['trigger']);
						$val['news_letter']  = self::mb_unserialize($val['news_letter']);
						$val['style']        = self::mb_unserialize($val['style']);

						$nnr_content_fire_db->update_data($val);

						$val 			= $ab_content_fire;
						$news_letter 	= self::mb_unserialize($val['news_letter']);
					}
				}
			}
		}

		// Include scripts

		self::generate_scripts($val);

		$template                 = self::mb_unserialize($val['template']);
		$trigger                  = self::mb_unserialize($val['trigger']);
		$style                 	  = self::mb_unserialize($val['style']);
		$args                	  = self::mb_unserialize($val['args']);
		$title                    = $val['title'];
		$text                     = $val['text'];
		$image                    = $val['image'];
		$aweber_customer_key      = $news_letter['aweber']['consumer_key'];
		$aweber_customer_secret   = $news_letter['aweber']['consumer_secret'];
		$aweber_access_key        = $news_letter['aweber']['access_key'];
		$aweber_access_secret     = $news_letter['aweber']['access_secret'];
		$aweber_account_id        = $news_letter['aweber']['account_id'];
		$aweber_list_id           = $news_letter['aweber']['list_id'];
		$feedburner_id            = $news_letter['feedburner']['id'];
		$news_letter_type         = $news_letter['type'];

		// Create Background Pattern credit to http://www.patternify.com

		$background = '';

		// Backgound Image

		if ( isset($style['background_type']) && $style['background_type'] == 'image' ) {

			$background_image_attributes = wp_get_attachment_image_src($style["background_image"], 'full'); // returns an array

			if( $background_image_attributes ) {
				$background = 'style="background-image: url(' . $background_image_attributes[0] . '); background-size: ' . $style['background_size'] . '; background-repeat: ' . $style['background_repeat'] . '; background-position: ' . $style['background_position'] . ';';
			}

		} else if ( isset($style['background_type']) && $style['background_type'] == 'pattern' ) {

			// Dots

			if ($style['background_pattern'] == 'dots') {
				$background = 'style="background: url(data:image/png;base64,iVBORw0KGgoAAAANSUhEUgAAAAoAAAAKCAYAAACNMs+9AAAAAXNSR0IArs4c6QAAAB1JREFUGBljYGBg4AFiQoAYNYTMGJWHhgAxgckDACfCADFzWBHCAAAAAElFTkSuQmCC);';
			}

			// Diagonal right

			else if ($style['background_pattern'] == 'diagonal_right') {
				$background = 'style="background: url(data:image/png;base64,iVBORw0KGgoAAAANSUhEUgAAAAoAAAAKCAYAAACNMs+9AAAAAXNSR0IArs4c6QAAABVJREFUGBljYCAO8BCjbFQRA/WCAABg/gB584t6RwAAAABJRU5ErkJggg==);';
			}

			// Diagonal left

			else if ($style['background_pattern'] == 'diagonal_left') {
				$background = 'style="background: url(data:image/png;base64,iVBORw0KGgoAAAANSUhEUgAAAAoAAAAKCAYAAACNMs+9AAAAAXNSR0IArs4c6QAAABhJREFUGBljYGBg4AFiosGoYuSgomFoAABg/gB5Q6KWYgAAAABJRU5ErkJggg==);';
			}

			// Grid

			else if ($style['background_pattern'] == 'grid') {
				$background = 'style="background: url(data:image/png;base64,iVBORw0KGgoAAAANSUhEUgAAAAoAAAAKCAYAAACNMs+9AAAAAXNSR0IArs4c6QAAAB1JREFUGBljZGBg4AFiogBRCpmIMgqoaFQh3pACADLgACqd10PPAAAAAElFTkSuQmCC);';
			}

			// Zig Zag

			else if ($style['background_pattern'] == 'zig_zag') {
				$background = 'style="background: url(data:image/png;base64,iVBORw0KGgoAAAANSUhEUgAAAAoAAAAICAYAAADA+m62AAAAAXNSR0IArs4c6QAAADFJREFUGBljYMAEPJhCmCIwRTAaUwVQBF0SnQ/WhFUQXTMuRTBrCcnD1OG1EUURaRwAYKwAecAGMfQAAAAASUVORK5CYII=);';
			}

			// Diamonds

			else if ($style['background_pattern'] == 'diamonds') {
				$background = 'style="background: url(data:image/png;base64,iVBORw0KGgoAAAANSUhEUgAAAAoAAAAKCAYAAACNMs+9AAAAAXNSR0IArs4c6QAAAEVJREFUGBljYMAEPEAhEMYLkBUgs1E0YZPAEMMQQDICLgdnIEmiM3mY0EUI8fGZiiGHIQA0HZsY2FJkCWQ2VheBFGAoAgCw1ADb2LiCOQAAAABJRU5ErkJggg==);';
			}

			// Hearts

			else if ($style['background_pattern'] == 'hearts') {
				$background = 'style="background: url(data:image/png;base64,iVBORw0KGgoAAAANSUhEUgAAAAoAAAAKCAYAAACNMs+9AAAAAXNSR0IArs4c6QAAADlJREFUGBljYKA14EGzAM5nRJKACyKJwZhfmGAsQjSyiSC12Ez9ApJAV4iuGKwIJIgLYDMZl1oyxQFOKAMqAtJ4RAAAAABJRU5ErkJggg==);';
			}

			// Circles

			else if ($style['background_pattern'] == 'circles') {
				$background = 'style="background: url(data:image/png;base64,iVBORw0KGgoAAAANSUhEUgAAAAoAAAAKCAYAAACNMs+9AAAAAXNSR0IArs4c6QAAADNJREFUGBljYKAH4AFaAsJ4AbICZDaKJmwScDEmFKUkcOAmAPUgs7EaAVJAUBFWnSQJAgBkhAB7H4o0xwAAAABJRU5ErkJggg==);';
			}

			// X Out

			else if ($style['background_pattern'] == 'x_out') {
				$background = 'style="background: url(data:image/png;base64,iVBORw0KGgoAAAANSUhEUgAAAAoAAAAKCAYAAACNMs+9AAAAAXNSR0IArs4c6QAAADRJREFUGBljYKAl4MFiODYxsDJkCWQ2FjMYGEAKMBQxYVVKhCCySchsFK3YJLCJoWginwMAZIQAe7Ze2SMAAAAASUVORK5CYII=);';
			}

			else {
				$background = 'style="';
			}
		}

		// Has Footer

		if ( isset($template['args']['footer']) && $template['args']['footer'] != '') {
			$background .= 'padding-bottom:0;"';
		} else {
			$background .= '"';
		}

		$content_fire_content = '';

		include(NNROBOTS_CONTENT_FIRE_PLUGIN_DIR . 'templates/content-fire.php');

		return $content_fire_content;
	}

	/**
	 * Adds the scripts and localizes the variables
	 *
	 * @since 1.0.0
	 *
	 * @param 	$val array of attrs
	 * @return 	N/A
	 */
	static function generate_scripts($val) {

		$newsletter	= self::mb_unserialize($val['news_letter']);
		$template 	= self::mb_unserialize($val['template']);
		$style    	= self::mb_unserialize($val['style']);
		$args    	= self::mb_unserialize($val['args']);

		wp_enqueue_style(
			NNRobots_Content_Fire::$prefix . 'template_' . $template['type'] . '_css',
			NNROBOTS_CONTENT_FIRE_PLUGIN_URL . '/templates/template-' . $template['type'] . '/template-' . $template['type'] . '.css'
		);

		wp_enqueue_style(
			NNRobots_Content_Fire::$prefix . 'css',
			NNROBOTS_CONTENT_FIRE_PLUGIN_URL . '/templates/content-fire.css'
		);

		wp_enqueue_style(
			NNRobots_Content_Fire::$prefix . 'fontawesome_css',
			NNROBOTS_CONTENT_FIRE_PLUGIN_URL . '/admin/fonts/font-awesome.min.css'
		);

		// Google Font Loader

		if ((isset($style['heading_google_fonts']) && $style['heading_google_fonts']) ||
			(isset($style['google_fonts']) && $style['google_fonts'])) {

			wp_enqueue_script(
				NNRobots_Content_Fire::$prefix . 'google_font_js',
				NNROBOTS_CONTENT_FIRE_PLUGIN_URL .  '/core/js/webfontloader.js'
			);

		}

		include_once(NNROBOTS_CONTENT_FIRE_PLUGIN_DIR . 'admin/functions.php');

		$default_content_fire = NNRobots_Content_Fire_Admin_Functions::default_content_fire();

	    $args_array = array(
			'prefix_dash'            => NNRobots_Content_Fire::$prefix_dash,
			'template'               => $template['type'],
			'preview'                => isset($_GET['action']) && $_GET['action'] == 'preview' ? true : false,
			'ajaxurl'                => admin_url( 'admin-ajax.php' ),
		);

		if (isset($_GET['action']) && $_GET['action'] == 'preview') {

			wp_enqueue_script(
				NNRobots_Content_Fire::$prefix . 'preview_js',
				NNROBOTS_CONTENT_FIRE_PLUGIN_URL . '/core/js/content-fire.js', array('jquery')
			);

			wp_localize_script(NNRobots_Content_Fire::$prefix . 'preview_js', 'nnr_cf_data', $args_array);

		} else {

			wp_enqueue_script(
				NNRobots_Content_Fire::$prefix . 'js',
				NNROBOTS_CONTENT_FIRE_PLUGIN_URL . '/core/js/content-fire.js', array('jquery')
			);

			wp_localize_script(NNRobots_Content_Fire::$prefix . 'js', 'nnr_cf_data', $args_array);
		}
	}

	/**
	 * AJAX funciton used to get all styling data for the content fire.
	 *
	 * @access public
	 * @static
	 * @return void
	 */
	static function get_styling_data() {

		// Check if content fire id is set

		if (!isset($_POST['content_fire_id']) || empty($_POST['content_fire_id'])) {
			echo '';
			die();
		}

		include_once(NNROBOTS_CONTENT_FIRE_PLUGIN_DIR . "/admin/model/db.php");
		include_once(NNROBOTS_CONTENT_FIRE_PLUGIN_DIR . "/admin/functions.php");

		global $wpdb;

		// Get all Content Fires

		$content_fire = $wpdb->get_results(
			$wpdb->prepare("SELECT * FROM `" . NNRobots_Content_Fire_DB::getTableName() . "` WHERE `id` = %d", $_POST['content_fire_id'])
		, 'ARRAY_A');

		$style = self::mb_unserialize($content_fire[0]['style']);

		$default_content_fire = NNRobots_Content_Fire_Admin_Functions::default_content_fire();

		echo json_encode(array(
			'id'					 => $_POST['content_fire_id'],
			'color'                  => isset($style['background_color']) ? $style['background_color'] : $default_content_fire['style']['background_color'],
			'border_color'           => isset($style['border_color']) ? $style['border_color'] : $default_content_fire['style']['border_color'],
			'text_color'             => isset($style['color']) ? $style['color'] : $default_content_fire['style']['color'],
			'heading_color'          => isset($style['heading_color']) ? $style['heading_color'] : $default_content_fire['style']['heading_color'],
			'nl_color'               => isset($style['nl_color']) ? $style['nl_color'] : $default_content_fire['style']['nl_color'],
			'nl_bg_color'            => isset($style['nl_bg_color']) ? $style['nl_bg_color'] : $default_content_fire['style']['nl_bg_color'],
			'cu_color'               => isset($style['cu_color']) ? $style['cu_color'] : $default_content_fire['style']['cu_color'],
			'cu_bg_color'            => isset($style['cu_bg_color']) ? $style['cu_bg_color'] : $default_content_fire['style']['cu_bg_color'],
			'yes_color'              => isset($style['yes_color']) ? $style['yes_color'] : $default_content_fire['style']['yes_color'],
			'yes_bg_color'           => isset($style['yes_bg_color']) ? $style['yes_bg_color'] : $default_content_fire['style']['yes_bg_color'],
			'no_color'               => isset($style['no_color']) ? $style['no_color'] : $default_content_fire['style']['no_color'],
			'no_bg_color'            => isset($style['no_bg_color']) ? $style['no_bg_color'] : $default_content_fire['style']['no_bg_color'],
			'font_size'              => isset($style['font_size']) ? $style['font_size'] : $default_content_fire['style']['font_size'],
	    	'font_weight'         	 => isset($style['font_weight']) ? $style['font_weight'] : $default_content_fire['style']['font_weight'],
	    	'google_fonts' 			 => isset($style['google_fonts']) ? $style['google_fonts'] : $default_content_fire['style']['google_fonts'],
	    	'heading_google_fonts' 	 => isset($style['heading_google_fonts']) ? $style['heading_google_fonts'] : $default_content_fire['style']['heading_google_fonts'],
	    	'font_family_heading' 	 => isset($style['font_family_heading']) ? $style['font_family_heading'] : $default_content_fire['style']['font_family_heading'],
	    	'font_family'         	 => isset($style['font_family']) ? $style['font_family'] : $default_content_fire['style']['font_family'],
		));

		die();
	}

	/**
	 * Checks that all content fire conditions are met
	 *
	 * @access public
	 * @static
	 * @param mixed $content_fire_id
	 * @param mixed $template
	 * @param mixed $args
	 * @param mixed $trigger
	 * @return void
	 */
	static function check_conditions($content_fire_id, $template, $args, $trigger){

		global $post;

		// Check if user has already signed up

		if ($args['show_after_success'] == 'no' &&
			in_array($template['type'], NNRobots_Content_Fire::$newsletter_templates) &&
			isset($_COOKIE[NNRobots_Content_Fire::$prefix . 'email_success_' . $content_fire_id])) {
			return false;
		}

		// Check if mobile

    	if (isset($args['mobile']) && $args['mobile'] == 'no' && wp_is_mobile()) {
    		return false;
		}

		// Check for excluded posts

		if ( is_singular() &&
			 isset($trigger[$post->post_type]['exclude']) &&
			 in_array($post->ID, explode(',', $trigger[$post->post_type]['exclude'])) ) {

    		return false;
		}

		// Check user role

		if (!self::check_user_role($trigger['args'])) {
    		return false;
		}

		// Check referrer

		if (!self::check_referrer($trigger['args'])) {
			return false;
		}

		// Date Range

		$now = time();

		if (isset($args['start_date']) && isset($args['end_date']) &&
			(strtotime($args['start_date']) > $now || strtotime($args['end_date']) < $now)) {
    		return false;
		}

		return true;
	}

	/**
	 * Determines if the referrer URL matches any given by the user
	 *
	 * @access public
	 * @static
	 * @param mixed $settings
	 * @return void
	 */
	static function check_referrer($settings) {

		// Return true any domain is checked

		if ($settings['referrer_type'] == 'any') {
			return true;
		}

		// Check is referrer domain matches ones given by user

		else if (isset($_SERVER["HTTP_REFERER"])) {

			$urls = explode(',', $settings['referrer']);
			$domain = parse_url($_SERVER["HTTP_REFERER"]);

			if (in_array($domain['host'], $urls)) {
				return true;
			}
		}

		return false;
	}

	/**
     * Check if the current user has access to backend / frontend based on his role compared with role from settings
     *
     * @access public
     * @static
     * @param mixed $settings
     * @return void
     */
    static function check_user_role($settings) {

		// Check if Everyone users is true

		if (isset($settings['users']) && $settings['users'] == 'everyone') {
			return true;
		}

		// Check if All users is true

		else if (isset($settings['users']) && $settings['users'] == 'logged_in' && is_user_logged_in()) {
			return true;
		}

		// Check if Logged Out Users is true

		else if (isset($settings['users']) && $settings['users'] == 'logged_out' && !is_user_logged_in()) {
			return true;
		}

		// Check if Specific User is true

		else if (isset($settings['users']) && $settings['users'] == 'specific' && is_user_logged_in()) {

			$is_allowed = false;
			global $wp_roles;
			$current_user = wp_get_current_user();

			foreach ($wp_roles->role_names as $role_name => $role) {

				if (isset($settings['user_roles'][$role_name]) && $settings['user_roles'][$role_name] && in_array($role_name, $current_user->roles)) {
					$is_allowed = true;
				}
			}

			return $is_allowed;
		}

        return false;
    }

    /**
	 * Mulit-byte Unserialize
	 *
	 * UTF-8 will screw up a serialized string
	 *
	 * @access private
	 * @param string
	 * @return string
	 */
	static function mb_unserialize($string) {

		if ( !is_string($string) ) {
			return $string;
		}

	    $string2 = preg_replace_callback(
	        '!s:(\d+):"(.*?)";!s',
	        function($m){
	            $len = strlen($m[2]);
	            $result = "s:$len:\"{$m[2]}\";";
	            return $result;

	        },
	        $string);
	    return unserialize($string2);
	}

}

//endif;